#include <mac/All.h>
#include <mac/MACLib.h>
#include <mac/APEInfo.h>
#include <mac/APETag.h>
#include <mac/CharacterHelper.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <wchar.h>

#include "common.h"

static IAPEDecompress *pAPEDecompress = NULL;
static char *current_filename = NULL;

static GtkWidget *window = NULL;
static GtkWidget *location = NULL;
static GtkWidget *metadata = NULL;
static GtkWidget *techinfo = NULL;

#ifdef __cplusplus
extern "C"{
#endif

static void insert_metadata(GtkCList* clist, CAPETag *pAPETag)
{
    if (!clist || !pAPETag)
    {
	return;
    }
    
    BOOL bHasID3Tag = pAPETag->GetHasID3Tag();
    BOOL bHasAPETag = pAPETag->GetHasAPETag();

    if (bHasID3Tag || bHasAPETag)
    {
	CAPETagField * pTagField;
	int index = 0;
	while ((pTagField = pAPETag->GetTagField(index)) != NULL)
	{
	    index ++;

	    if (pTagField->GetFieldValueSize() > 128)
	    {
	    }
	    else
	    {
		const wchar_t *fieldName;
		char *name;
		const char *fieldValue;
		char *value;

		fieldName = pTagField->GetFieldName();
		name = GetANSIFromUTF16(fieldName);

		fieldValue = pTagField->GetFieldValue();
		if (pAPETag->GetAPETagVersion() == CURRENT_APE_TAG_VERSION)
		{
		    value = GetANSIFromUTF8((unsigned char *)fieldValue);
		}
		else
		{
		    value = (char *)fieldValue;
		}

		gchar *data[2];
		data[0] = g_strdup(name);
		data[1] = g_strdup(value);

		g_free(name);
		if (pAPETag->GetAPETagVersion() == CURRENT_APE_TAG_VERSION)
		{
		    g_free(value);
		}

		gtk_clist_append(clist, data);

		g_free(data[0]);
		g_free(data[1]);
	    }
	}
    }
}

static void insert_techinfo(GtkCList* clist, IAPEDecompress *pAPEDecompress)
{
    if (!clist || !pAPEDecompress)
    {
	return;
    }

    gchar *data[2];

    /* APE Version */
    data[0] = _("Version");
    data[1] = g_strdup_printf("%.2f", (float)pAPEDecompress->GetInfo(APE_INFO_FILE_VERSION) / float(1000));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Compression Level */
    data[0] = _("Compression Level");
    switch (pAPEDecompress->GetInfo(APE_INFO_COMPRESSION_LEVEL))
    {
    case COMPRESSION_LEVEL_FAST:
	data[1] = _("Fast");
	break;
    case COMPRESSION_LEVEL_NORMAL:
	data[1] = _("Normal");
	break;
    case COMPRESSION_LEVEL_HIGH:
	data[1] = _("High");
	break;
    case COMPRESSION_LEVEL_EXTRA_HIGH:
	data[1] = _("Extra High");
	break;
    case COMPRESSION_LEVEL_INSANE:
	data[1] = _("Insane");
	break;
    }
    gtk_clist_append(clist, data);

    /* Format Flags */
    data[0] = _("Flags");
    data[1] = g_strdup_printf("%d", pAPEDecompress->GetInfo(APE_INFO_FORMAT_FLAGS));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Sample Rate */
    data[0] = _("Sample Rate");
    data[1] = g_strdup_printf("%d", pAPEDecompress->GetInfo(APE_INFO_SAMPLE_RATE));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Bits Per Sample */
    data[0] = _("Bits Per Sample");
    data[1] = g_strdup_printf("%d", pAPEDecompress->GetInfo(APE_INFO_BITS_PER_SAMPLE));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Channels */
    data[0] = _("Channels");
    data[1] = g_strdup_printf("%d", pAPEDecompress->GetInfo(APE_INFO_CHANNELS));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Actrual File if the file is APL */
    char *ext = strrchr(current_filename, '.');
    if(ext)
    {
	if (!strcasecmp(ext, ".apl"))
	{
	    data[0] = _("Actrual File");
	    CIO *pIO = (CIO *)pAPEDecompress->GetInfo(APE_INFO_IO_SOURCE);
	    if (pIO)
	    {
		wchar_t nameBuf[255];
		memset(nameBuf, 0, 255 * sizeof(wchar_t));
		pIO->GetName(nameBuf);
		char *name = GetANSIFromUTF16(nameBuf);
		char *basename = g_basename(name);
		data[1] = basename;
		gtk_clist_append(clist, data);
		g_free(name);
	    }
	}
    }
    
    /* Total File Size */
    data[0] = _("File Size");
    data[1] = g_strdup_printf("%d bytes", pAPEDecompress->GetInfo(APE_INFO_APE_TOTAL_BYTES));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Length (Time) */
    data[0] = _("Length");
    unsigned int length = pAPEDecompress->GetInfo(APE_DECOMPRESS_LENGTH_MS);
    int min, sec, ms;
    ms = length % 1000;
    sec = length / 1000;
    min = sec / 60;
    sec = sec % 60;
    data[1] = g_strdup_printf("%d:%d.%d ", min, sec, ms);
    gtk_clist_append(clist, data);
    g_free(data[1]);

    /* Average Bitrate */
    data[0] = _("Bitrate");
    data[1] = g_strdup_printf("%d", pAPEDecompress->GetInfo(APE_DECOMPRESS_AVERAGE_BITRATE));
    gtk_clist_append(clist, data);
    g_free(data[1]);

    
}

void mac_file_info_box(char *filename)
{
    char *title;

    int nRetVal;
    wchar_t *pUTF16Name;

    if (!window)
    {
	GtkWidget *location_frame, *metadata_frame, *techinfo_frame;
	GtkWidget *vbox, *hbox;
	GtkWidget *bottom_hbox;
	GtkWidget *close_button;
	
	window = gtk_window_new(GTK_WINDOW_DIALOG);
//	gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);

	gtk_signal_connect(GTK_OBJECT(window), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);

	gtk_container_set_border_width(GTK_CONTAINER(window), 10);

	location_frame = gtk_frame_new(_("Location"));
	metadata_frame = gtk_frame_new(_("Metadata"));
	techinfo_frame = gtk_frame_new(_("Technical info"));

	vbox = gtk_vbox_new(FALSE, 5);
	hbox = gtk_hbox_new(FALSE, 5);
	bottom_hbox = gtk_hbox_new(FALSE, 0);

	gtk_container_add(GTK_CONTAINER(window), vbox);

	gtk_box_pack_start(GTK_BOX(vbox), location_frame, 
			   FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox,
			   TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), bottom_hbox,
			   FALSE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(hbox), metadata_frame,
			   TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), techinfo_frame,
			   FALSE, FALSE, 0);

	close_button = gtk_button_new_with_label(_("Close"));
	gtk_signal_connect_object (GTK_OBJECT(close_button), "clicked",
				 GTK_SIGNAL_FUNC(gtk_widget_destroy),
				 GTK_OBJECT(window));
	gtk_box_pack_end(GTK_BOX(bottom_hbox), close_button,
			 TRUE, FALSE, 0);

	location = gtk_entry_new();
	gtk_container_add(GTK_CONTAINER(location_frame), location);

	gchar *column[2] = {_("Name"), _("Value")};
	metadata = gtk_clist_new_with_titles(2, column);
//	metadata = gtk_clist_new(2);
	gtk_clist_column_titles_passive(GTK_CLIST(metadata));
	gtk_clist_set_column_width(GTK_CLIST(metadata), 0, 70);
	gtk_clist_set_column_width(GTK_CLIST(metadata), 1, 160);

	gtk_container_add(GTK_CONTAINER(metadata_frame), metadata);

	techinfo = gtk_clist_new_with_titles(2, column);
//	techinfo = gtk_clist_new(2);
	gtk_clist_column_titles_passive(GTK_CLIST(techinfo));
	gtk_clist_set_column_width(GTK_CLIST(techinfo), 0, 130);
	gtk_clist_set_column_width(GTK_CLIST(techinfo), 1, 100);

	gtk_container_add(GTK_CONTAINER(techinfo_frame), techinfo);

	gtk_widget_show_all(window);
    }
    
    if (current_filename)
    {
	g_free(current_filename);
    }
    current_filename = g_strdup(filename);
    
    title = g_strdup_printf(_("File Info - %s"), g_basename(filename));
    gtk_window_set_title(GTK_WINDOW(window), title);
    g_free(title);
    
    gtk_entry_set_text(GTK_ENTRY(location), filename);
    gtk_entry_set_editable(GTK_ENTRY(location), FALSE);

    if (pAPEDecompress)
    {
	delete pAPEDecompress;
    }

    pUTF16Name = GetUTF16FromANSI(filename);
    pAPEDecompress = CreateIAPEDecompress(pUTF16Name, &nRetVal);

    if (pAPEDecompress == NULL || nRetVal != ERROR_SUCCESS)
    {
	printf("Oops, something is wrong with the file\n");
	return;
    }

    CAPETag *apeTag = (CAPETag *)pAPEDecompress->GetInfo(APE_INFO_TAG);
    insert_metadata(GTK_CLIST(metadata), apeTag);
    insert_techinfo(GTK_CLIST(techinfo), pAPEDecompress);

    delete pAPEDecompress;
    pAPEDecompress = NULL;
}

int main(int argc, char *argv[])
{
    if (argc < 2)
    {
	printf("usage: fileinfo filename\n");
	return 0;
    }
    gtk_set_locale();
    gtk_init(&argc, &argv);

    mac_file_info_box(argv[1]);

    gtk_signal_connect(GTK_OBJECT (window), "delete_event",
                       (GtkSignalFunc) gtk_exit, NULL);

    gtk_main();
    
    return 0;
}

#ifdef __cplusplus
}
#endif
